Rock = GameObject:extend()

function Rock:new(area, x, y, opts)
    Rock.super.new(self, area, x, y, opts)

    local direction = opts.direction or table.random({-1, 1})
    self.x = opts.x or (gw/2 + direction*(gw/2 + 48))
    self.y = opts.y or (random(16, gh - 16))

    self.shape = HC.polygon(unpack(createIrregularPolygon(8)))
    self.shape.id = self.id
    self.shape:moveTo(self.x, self.y)
    self.shape.object = self
    self.shape.tag = 'Enemy'

    self.w, self.h = 8, 8

    self.v = -direction*random(20, 40)
    self.vx = self.v
    self.dr = random(-10, 10)

    self.hp = 100*current_room.director.enemy_hp_multiplier
end

function Rock:update(dt)
    Rock.super.update(self, dt)

    self:enemyProjectileCollisions()

    self.shape:move(self.vx*dt, 0)
    self.shape:rotate(self.dr*dt)
    self.x, self.y = self.shape:center()
end

function Rock:draw()
    love.graphics.setColor(color255To1(hp_color))
    if self.hit_flash then love.graphics.setColor(color255To1(default_color)) end
    self.shape:draw('line')
    love.graphics.setColor(color255To1(default_color))
end

function Rock:destroy()
    Rock.super.destroy(self)
end

function Rock:hit(damage)
    if self.dead then return end

    self.hp = self.hp - (damage or 100)
    if self.hp <= 0 then
        playGameEnemyDie()
        current_room.score = current_room.score + 100
        self.dead = true
        if not current_room.player.no_ammo_drop then self.area:addGameObject('Ammo', self.x, self.y) end
        self.area:addGameObject('EnemyDeathEffect', self.x, self.y, {color = hp_color, w = 3*self.w})
    else
        self.hit_flash = true
        self.timer:after('hit_flash', 0.2, function() self.hit_flash = false end)
    end
end
